/* The GIMP -- an image manipulation program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <stdlib.h>
#include <stdio.h>
#include <sys/param.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <pwd.h>
#include <grp.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#ifdef __EMX__
#  include <process.h>
#endif
#include "gtk/gtk.h"
#include "libgimp/gimp.h"
#include "libgimp/gimptypes.h"
#include "plugin-intl.h"

#define DEBUG
#ifdef DEBUG
# define dbgprint(x)	printf x
#else
# define dbgprint(x)	""
#endif

/* Author: Martin Edlman. */

static gint show_info      (GimpParam * values);
static void ok_callback    (GtkWidget * widget, gpointer data);
static void close_callback (GtkWidget * widget, gpointer data);
static void query       (void);
static void run         (const char      *name,
                         gint             nparams,
                         const GimpParam *param,
                         gint            *nreturn_vals,
                         GimpParam      **return_vals);

GimpPlugInInfo PLUG_IN_INFO =
{
  NULL,    /* init_proc */
  NULL,    /* quit_proc */
  query,   /* query_proc */
  run,     /* run_proc */
};

MAIN ()

static void query () {
  static GimpParamDef input_args[] = {
    { GIMP_PDB_INT32,  "run_mode",    "Interactive, non-interactive" },
    { GIMP_PDB_IMAGE,  "image",       "Input image" },
    { GIMP_PDB_DRAWABLE,"drawable",   "Input drawable - not used" },
    { GIMP_PDB_INT32,  "sizemod",     "Size modifier {0-B,1-KB,2-MB,3-GB,4-TB}" },
    { GIMP_PDB_INT32,  "sizeprec",    "Size decimal precision (e.g. 2 for 1.49)" },
    { GIMP_PDB_INT32,  "rightsfmt",   "Access rights format {0-octal,1-letters}" },
    { GIMP_PDB_INT32,  "ownerfmt",    "Owner format {0-uid,1-name}" },
    { GIMP_PDB_INT32,  "groupfmt",    "Group format {0-gid,1-name}" },
    { GIMP_PDB_STRING, "datetimefmt", "Format for atime,mtime,ctime (see 'man strftime')" },
  };

  static GimpParamDef output_vals[] = {
    { GIMP_PDB_STRING, "fullname", "Full filename (path+name+extension)" },
    { GIMP_PDB_STRING, "path",     "Path to file" },
    { GIMP_PDB_STRING, "name",     "Filename" },
    { GIMP_PDB_STRING, "ext",      "Extension" },
    { GIMP_PDB_STRING, "size",     "Size of the file according to 'sizemod'" },
    { GIMP_PDB_STRING, "rights",   "Access rights to the file" },
    { GIMP_PDB_STRING, "owner",    "Owner of the file" },
    { GIMP_PDB_STRING, "group",    "Group of the file" },
    { GIMP_PDB_STRING, "atime",    "Date/time of last access" },
    { GIMP_PDB_STRING, "mtime",    "Date/time of last modification" },
    { GIMP_PDB_STRING, "ctime",    "Date/time of last change" },
  };

  static int ninput_args  = sizeof (input_args) / sizeof (input_args[0]);
  static int noutput_vals = sizeof (output_vals) / sizeof (output_vals[0]);

//  INIT_LOCALE();

  dbgprint(("fileinfo: query: calling gimp_install_procedure\n"));
  gimp_install_procedure ("plug_in_fileinfo_image",
                          _("Returns miscelanous image file information"),
                          _("This function gives you some information about file \
like its location, size, access rights, owner and group, change, \
modification and access times.\n\
This works only on opened images, if someone make it working on files \
specified by filename (not only images) it should be usefull for making \
image indexes of directories.\n\
I started on it but now I don't have a time to finish it. My idea is to do \
everything in plug_in_fileinfo_filename which should be called from here \
after obtaining filename of this image."),
                          "Martin Edlman",
                          "Martin Edlman",
                          "16/02/2000 - v.0.1.1",
                          N_("<Image>/File/Information"),
			  "RGB*,GRAY*,INDEXED*",
                          GIMP_PLUGIN,
                          ninput_args, noutput_vals,
                          input_args, output_vals);
  dbgprint(("fileinfo: query: leaving\n"));
}

static void run (const char      *name,
                 gint             nparams,
                 const GimpParam *param,
                 gint            *nreturn_vals,
                 GimpParam      **return_vals) {
                                  // PASSED PARAMETERS
    GimpRunMode run_mode;         // self explaining
    gint32   sizemod;             // size modifier
    gint32   sizeprec;            // size precision
    gint32   modefmt;             // access rights format
    gint32   userfmt;             // user format
    gint32   groupfmt;            // group format
    gchar   *datetimefmt;         // date and/or time format
    static GimpParam values[12];  // RETURNED VALUES
    gchar   *fullname;            // obtained by other gimp function
    gchar   *filepath;            // allocated dynamicaly later
    gchar   *filename;            // -"-
    gchar   *fileext;             // -"-
    gchar   *mode;                // -"-
    gchar   *user;                // -"-
    gchar   *group;               // -"-
    gchar   *size;                // -"-
    gchar   *ctime;               // -"-
    gchar   *atime;               // -"-
    gchar   *mtime;               // -"-
                                  // SUPPORT VARIABLES AND STRUCTURES
    gchar         *fnsplit;       // filename split
    gchar          buff[1000];    // buffer for string ops
    gchar          sizefmt[10];   // string containing size formating string
    gfloat         sizenum;       // numeric size representation
    struct stat    filestat;      // file stat info
    struct passwd *pwent;         // password file entry for user name
    struct group  *grent;         // group file entry for group name

#ifdef DEBUG
    int    debug = 0;
#endif

//    INIT_LOCALE();

    run_mode = param[0].data.d_int32;

    if(run_mode == GIMP_RUN_NONINTERACTIVE) {
        sizemod  = param[3].data.d_int32;
        sizeprec = param[4].data.d_int32;
        modefmt  = param[5].data.d_int32;
        userfmt  = param[6].data.d_int32;
        groupfmt = param[7].data.d_int32;
        datetimefmt = param[8].data.d_string;
    }
    else {
        sizemod  = 0;                // bytes
        sizeprec = 0;                // no precision when counting bytes
        modefmt  = 1;                // verbose 'rwxrwxrwx'
        userfmt  = 1;                // user name (not uid)
        groupfmt = 1;                // group name (not gid)
        datetimefmt = "%b %d %Y %T"; // Jan 01 1999 01:59:34
    }

    *nreturn_vals = 11;
    *return_vals = values;

    dbgprint(("*** DEBUG INFO: %i\n", ++debug));

    values[0].type = GIMP_PDB_STATUS;
    values[0].data.d_status = GIMP_PDB_CALLING_ERROR;

    dbgprint(("*** DEBUG INFO: %i\n", ++debug));
    if(strcmp(name, "plug_in_fileinfo_image") == 0) {
        dbgprint(("*** DEBUG INFO: %i\n+++ Image ID: %i\n", ++debug, param[1].data.d_image));
        fullname = gimp_image_get_filename(param[1].data.d_image);
        dbgprint(("*** DEBUG INFO: %i\n--- Fullname: %s\n", ++debug, fullname));
        if(fullname != NULL && strncmp(fullname, "Untitled", 8) != 0) {

            // FILE PATH, NAME, EXTENSION
            filepath = malloc(strlen(fullname) + 1);
            strcpy(filepath, fullname);
            fnsplit = strrchr(filepath, '.');
            fileext = &fnsplit[1];
            fnsplit[0] = '\0';
            fnsplit = strrchr(filepath, '/');
            filename = &fnsplit[1];
            fnsplit[0] = '\0';

            stat(fullname, &filestat);

            //SIZE
            sizenum = filestat.st_size;
            switch (sizemod) {                 // size
            case 4: sizenum /= 1024;           // terabytes
            case 3: sizenum /= 1024;           // gigabytes
            case 2: sizenum /= 1024;           // megabytes
            case 1: sizenum /= 1024;           // kilobytes
            case 0: break;
            }
            snprintf(sizefmt, sizeof(sizefmt) - 1, "%%.%if", sizeprec);
            snprintf(buff, sizeof(buff) - 1, sizefmt, sizenum);
            size = malloc(strlen(buff) + 1);
            strcpy(size, buff);
            dbgprint(("*** DEBUG INFO: %i\n+++ Size mod: %i\n--- Size num: %.5f\n--- Size fmt: %s\n--- Size: %s\n", ++debug, sizemod, sizenum, sizefmt, size));

            // ACCESS RIGHTS
            if(modefmt == 0)                     // mode - octal
                snprintf(buff, sizeof(buff) - 1, "%o", filestat.st_mode);
            else {                              // mode - verbose
                strncpy(buff, "---------", sizeof(buff) - 1);
                if(filestat.st_mode & 256) buff[0] = 'r';  // user read   (400 oct = 256 dec)
                if(filestat.st_mode & 128) buff[1] = 'w';  // user write  (200 oct = 128 dec)
                if(filestat.st_mode &  64) buff[2] = 'x';  // user exec   (100 oct =  64 dec)
                if(filestat.st_mode &  32) buff[3] = 'r';  // group read  (040 oct =  32 dec)
                if(filestat.st_mode &  16) buff[4] = 'w';  // group write (020 oct =  16 dec)
                if(filestat.st_mode &   8) buff[5] = 'x';  // group exec  (010 oct =   8 dec)
                if(filestat.st_mode &   4) buff[6] = 'r';  // other read  (004 oct =   4 dec)
                if(filestat.st_mode &   2) buff[7] = 'w';  // other write (002 oct =   2 dec)
                if(filestat.st_mode &   1) buff[8] = 'x';  // other exec  (001 oct =   1 dec)
            }
            mode = malloc(strlen(buff) + 1);
            strcpy(mode, buff);
            dbgprint(("*** DEBUG INFO: %i\n+++ Mode fmt: %i\n--- Mode: %s\n", ++debug, modefmt, mode));

            // USER
            if(userfmt == 0)                     // user - uid
                snprintf(buff, sizeof(buff) - 1, "%i", filestat.st_uid);
            else {                               // user - name
                pwent = getpwuid(filestat.st_uid);
                snprintf(buff, sizeof(buff) - 1, "%s", pwent->pw_name);
            }
            user = malloc(strlen(buff) + 1);
            strcpy(user, buff);
            dbgprint(("*** DEBUG INFO: %i\nUser: %s\n", ++debug, user));

            // GROUP
            if(groupfmt == 0)                    // group - gid
                snprintf(buff, sizeof(buff) - 1, "%i", filestat.st_gid);
            else {                               // group - name
                grent = getgrgid(filestat.st_gid);
                snprintf(buff, sizeof(buff) - 1, "%s", grent->gr_name);
            }
            group = malloc(strlen(buff) + 1);
            strcpy(group, buff);
            dbgprint(("*** DEBUG INFO: %i\nGroup: %s\n", ++debug, group));

            // DATES AND/OR TIMES
            strftime(buff, sizeof(buff) - 1, datetimefmt, gmtime(&filestat.st_ctime));
            ctime = malloc(strlen(buff) + 1);
            strcpy(ctime, buff);
            dbgprint(("*** DEBUG INFO: %i\nCtime: %s\n", ++debug, ctime));
            strftime(buff, sizeof(buff) - 1, datetimefmt, gmtime(&filestat.st_atime));
            atime = malloc(strlen(buff) + 1);
            strcpy(atime, buff);
            dbgprint(("*** DEBUG INFO: %i\nAtime: %s\n", ++debug, atime));
            strftime(buff, sizeof(buff) - 1, datetimefmt, gmtime(&filestat.st_mtime));
            mtime = malloc(strlen(buff) + 1);
            strcpy(mtime, buff);
            dbgprint(("*** DEBUG INFO: %i\nMtime: %s\n", ++debug, mtime));

            dbgprint(("*** DEBUG INFO: %i\n", ++debug));
            *nreturn_vals = 12;

            values[0].data.d_status = GIMP_PDB_SUCCESS;
            values[1].type = GIMP_PDB_STRING;
            values[1].data.d_string = fullname;
            values[2].type = GIMP_PDB_STRING;
            values[2].data.d_string = filepath;
            values[3].type = GIMP_PDB_STRING;
            values[3].data.d_string = filename;
            values[4].type = GIMP_PDB_STRING;
            values[4].data.d_string = fileext;
            values[5].type = GIMP_PDB_STRING;
            values[5].data.d_string = size;
            values[6].type = GIMP_PDB_STRING;
            values[6].data.d_string = mode;
            values[7].type = GIMP_PDB_STRING;
            values[7].data.d_string = user;
            values[8].type = GIMP_PDB_STRING;
            values[8].data.d_string = group;
            values[9].type = GIMP_PDB_STRING;
            values[9].data.d_string = atime;
            values[10].type = GIMP_PDB_STRING;
            values[10].data.d_string = mtime;
            values[11].type = GIMP_PDB_STRING;
            values[11].data.d_string = ctime;

            dbgprint(("*** DEBUG INFO: %i\n", ++debug));
            if(run_mode == GIMP_RUN_INTERACTIVE)      // interactive
                show_info(values);
        }
        else {
            dbgprint(("*** DEBUG INFO: %i\n", ++debug));
            g_message("Image has not filename yet\nSAVE IT FIRST!\n");
            values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
        }
    }
    else
        g_assert (FALSE);
}

static gint show_info(GimpParam *values) {
    GtkWidget *_dlg;
    GtkWidget *_okbutton;
    GtkWidget *_table;
    GtkWidget *_label;

    gint       counter;
    gchar     *sizestr;
    gchar     *labels[] = {
        _("Fullname:"), _("Path:"), _("Name:"), _("Extension:"),
        _("Size:"), _("Access rights:"), _("Owner:"), _("Group:"),
        _("Last access:"), _("Last modification:"), _("Last change:")
    };

    gint    argc;
    gchar **argv;

    argc = 1;
    argv = g_new (gchar *, 1);
    argv[0] = g_strdup ("fileinfo");

    gtk_init (&argc, &argv);
    gtk_rc_parse (gimp_gtkrc ());

    _dlg = gtk_dialog_new ();
    gtk_window_set_title (GTK_WINDOW (_dlg), _("File information"));
    gtk_signal_connect (GTK_OBJECT (_dlg), "destroy",
                        (GtkSignalFunc) close_callback, NULL);

    _okbutton = gtk_button_new_with_label (_("OK"));
    GTK_WIDGET_SET_FLAGS (_okbutton, GTK_CAN_DEFAULT);
    gtk_signal_connect (GTK_OBJECT (_okbutton), "clicked",
                        (GtkSignalFunc) ok_callback,
                        _dlg);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (_dlg)->action_area), _okbutton, TRUE, TRUE, 0);
    gtk_widget_grab_default (_okbutton);
    gtk_widget_show (_okbutton);

    _table = gtk_table_new (11, 2, FALSE);
    gtk_container_set_border_width (GTK_CONTAINER (_table), 10);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (_dlg)->vbox), _table, FALSE, FALSE, 0);
    gtk_widget_show (_table);

    gtk_table_set_row_spacings (GTK_TABLE (_table), 2);
    gtk_table_set_col_spacings (GTK_TABLE (_table), 10);

    for (counter = 1; counter <= 11; counter++) {
        _label = gtk_label_new (labels[counter - 1]);
        gtk_misc_set_alignment (GTK_MISC (_label), 1, 0.5);
        gtk_table_attach (GTK_TABLE (_table), _label,
                          0, 1, counter - 1, counter,
                          GTK_EXPAND | GTK_FILL,
                          GTK_EXPAND | GTK_FILL,
                          0, 0);
        if (counter == 5) {       // size - append modifier
            sizestr = malloc (strlen(values[counter].data.d_string) + 9);
            sprintf (sizestr, _("%s byte(s)"), values[counter].data.d_string);
            _label = gtk_label_new (sizestr);
            free(sizestr);
        }
        else
            _label = gtk_label_new (values[counter].data.d_string);
        gtk_misc_set_alignment (GTK_MISC (_label), 0, 0.5);
        gtk_table_attach (GTK_TABLE (_table), _label,
                          1, 2, counter - 1, counter,
                          GTK_EXPAND | GTK_FILL,
                          GTK_EXPAND | GTK_FILL,
                          0, 0);
    }

    gtk_widget_show_all (_dlg);
    gtk_main ();
    gdk_flush ();
    return 1;
}

static void ok_callback (GtkWidget * widget, gpointer data) {
    gtk_widget_destroy (GTK_WIDGET (data));
}

static void close_callback (GtkWidget * widget, gpointer data) {
    gtk_main_quit ();
}
