/* The GIMP -- an image manipulation program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <stdlib.h>
#include <stdio.h>
#include <sys/param.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <pwd.h>
#include <grp.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#ifdef __EMX__
#include <process.h>
#endif
#include "libgimp/gimp.h"

#define DEBUG

/* Author: Martin Edlman. */

static void   query      (void);
static void   run        (char    *name,
                          int      nparams,
                          GParam  *param,
                          int     *nreturn_vals,
                          GParam **return_vals);

GPlugInInfo PLUG_IN_INFO =
{
  NULL,    /* init_proc */
  NULL,    /* quit_proc */
  query,   /* query_proc */
  run,     /* run_proc */
};

MAIN ()

static void query () {
  static GParamDef input_args[] = {
    { PARAM_INT32,  "run_mode",    "Interactive, non-interactive" },
    { PARAM_STRING, "filename",    "File name" },
    { PARAM_INT32,  "sizefmt",     "Size modifier {0-B,1-KB,2-MB,3-GB,4-TB}" },
    { PARAM_INT32,  "modefmt",     "Access rights format {0-octal,1-letters}" },
    { PARAM_INT32,  "userfmt",     "Owner format {0-uid,1-name}" },
    { PARAM_INT32,  "groupfmt",    "Group format {0-gid,1-name}" },
    { PARAM_STRING, "datetimefmt", "Format for cdate,adate,mdate (see 'man strftime')" },
  };

  static GParamDef output_vals[] = {
    { PARAM_STRING, "fullname", "Full filename (path+name+extension)" },
    { PARAM_STRING, "path",     "Path to file" },
    { PARAM_STRING, "name",     "Filename" },
    { PARAM_STRING, "ext",      "Extension" },
    { PARAM_STRING, "size",     "Size of the file according to 'sizefmt'" },
    { PARAM_STRING, "mode",     "Access rights to the file" },
    { PARAM_STRING, "user",     "Owner of the file" },
    { PARAM_STRING, "group",    "Group of the file" },
    { PARAM_STRING, "atime",    "Last access date/time of the file" },
    { PARAM_STRING, "mtime",    "Last modification date/time of the file" },
    { PARAM_STRING, "ctime",    "Creation date/time of file" },
  };

  static int ninput_args  = sizeof (input_args) / sizeof (input_args[0]);
  static int noutput_vals = sizeof (output_vals) / sizeof (output_vals[0]);

#ifdef DEBUG
  printf("fileinfo: query: calling gimp_install_procedure\n");
#endif
  gimp_install_procedure ("file_info_name",
                          "returns miscelanous file information",
                          "This function gives you some information about file \
specified by its full filename (path+filename+extension) \
like its location, size, access rights, owner and group, \
creation, modification and access times.",
                          "Martin Edlman",
                          "Martin Edlman",
                          "1999",
                          "<Toolbox>/File/Information",
			  "RGB*,GRAY*,INDEXED*",
                          PROC_PLUG_IN,
                          ninput_args, noutput_vals,
                          input_args, output_vals);
#ifdef DEBUG
  printf("fileinfo: query: leaving\n");
#endif
}

static void run (char *name, int nparams, GParam *param, int *nreturn_vals,
                 GParam **return_vals) {
  static GParam values[12];
  GRunModeType run_mode;
  gchar   *fullname;
  gchar   *filepath, *filename, *fileext;
  gchar   *mode, user[100], group[100], size[100];
  gchar    ctime[500], atime[500], mtime[500];
  gint32   modefmt, sizefmt, userfmt, groupfmt;
  gchar   *datetimefmt;
  struct stat    filestat;
  struct passwd *pwent;
  struct group  *grent;
  char          *fnsplit;  // filename split

#ifdef DEBUG
  int    debug = 0;
#endif

  run_mode = param[0].data.d_int32;

  if(run_mode == RUN_NONINTERACTIVE) {
    fullname = param[1].data.d_string;
    sizefmt  = param[2].data.d_int32;
    modefmt  = param[3].data.d_int32;
    userfmt  = param[4].data.d_int32;
    groupfmt = param[5].data.d_int32;
    datetimefmt = param[6].data.d_string;
  }
  else {
    fullname = NULL;
    sizefmt  = 0;                // bytes
    modefmt  = 1;                // verbose 'rwxrwxrwx'
    userfmt  = 1;                // user name (not uid)
    groupfmt = 1;                // group name (not gid)
    datetimefmt = "%b %d %Y %T"; // Jan 01 1999 01:59:34
  }

  *nreturn_vals = 11;
  *return_vals = values;

#ifdef DEBUG
  printf("*** DEBUG INFO: %i\n", ++debug);
#endif

  values[0].type = PARAM_STATUS;
  values[0].data.d_status = STATUS_CALLING_ERROR;

#ifdef DEBUG
  printf("*** DEBUG INFO: %i\n", ++debug);
#endif
  if(strcmp(name, "file_info_name") == 0) {
#ifdef DEBUG
      printf("*** DEBUG INFO: %i\n--- Fullname: %s\n", ++debug, fullname);
#endif
      if(fullname != NULL && strncmp(fullname, "Untitled", 8) != 0) {
          filepath = malloc(strlen(fullname) + 1);
          strncpy(filepath, fullname, strlen(fullname));
          strcat(filepath, "\0");
          fnsplit = strrchr(filepath, '.');
          fileext = &fnsplit[1];
          fnsplit[0] = '\0';
          fnsplit = strrchr(filepath, '/');
          filename = &fnsplit[1];
          fnsplit[0] = '\0';
          stat(fullname, &filestat);
          switch (sizefmt) {                   // size
          case 4: filestat.st_size /= 1024;  // terabytes
          case 3: filestat.st_size /= 1024;  // gigabytes
          case 2: filestat.st_size /= 1024;  // megabytes
          case 1: filestat.st_size /= 1024;  // kilobytes
          case 0: break;
          }
          snprintf(size, sizeof(size) - 1, "%li", filestat.st_size);
#ifdef DEBUG
          printf("*** DEBUG INFO: %i\n+++ Size fmt: %i\n--- Size: %s\n", ++debug, sizefmt, size);
#endif

          mode = malloc(10);
          if(modefmt == 0)                     // mode - octal
              sprintf(mode, "%o", filestat.st_mode);
          else {                              // mode - verbose
              strcpy(mode, "---------");
              if(filestat.st_mode & 256) mode[0] = 'r';  // user read   (400 oct = 256 dec)
              if(filestat.st_mode & 128) mode[1] = 'w';  // user write  (200 oct = 128 dec)
              if(filestat.st_mode &  64) mode[2] = 'x';  // user exec   (100 oct =  64 dec)
              if(filestat.st_mode &  32) mode[3] = 'r';  // group read  (040 oct =  32 dec)
              if(filestat.st_mode &  16) mode[4] = 'w';  // group write (020 oct =  16 dec)
              if(filestat.st_mode &   8) mode[5] = 'x';  // group exec  (010 oct =   8 dec)
              if(filestat.st_mode &   4) mode[6] = 'r';  // other read  (004 oct =   4 dec)
              if(filestat.st_mode &   2) mode[7] = 'w';  // other write (002 oct =   2 dec)
              if(filestat.st_mode &   1) mode[8] = 'x';  // other exec  (001 oct =   1 dec)
          }

#ifdef DEBUG
          printf("*** DEBUG INFO: %i\n+++ Mode fmt: %i\n--- Mode: %s\n", ++debug, modefmt, mode);
#endif
          if(userfmt == 0)                     // user - uid
              snprintf(user, sizeof(user) - 1, "%i", filestat.st_uid);
          else {                               // user - name
              pwent = getpwuid(filestat.st_uid);
              snprintf(user, sizeof(user) - 1, "%s", pwent->pw_name);
          }

#ifdef DEBUG
          printf("*** DEBUG INFO: %i\nUser: %s\n", ++debug, user);
#endif
          if(groupfmt == 0)                    // group - gid
              snprintf(group, sizeof(group) - 1, "%i", filestat.st_gid);
          else {                               // group - name
              grent = getgrgid(filestat.st_gid);
              snprintf(group, sizeof(group) - 1, "%s", grent->gr_name);
          }

#ifdef DEBUG
          printf("*** DEBUG INFO: %i\nGroup: %s\n", ++debug, group);
#endif
          strftime(ctime, sizeof(ctime) - 1, datetimefmt, gmtime(&filestat.st_ctime));
          strftime(atime, sizeof(atime) - 1, datetimefmt, gmtime(&filestat.st_atime));
          strftime(mtime, sizeof(mtime) - 1, datetimefmt, gmtime(&filestat.st_mtime));

#ifdef DEBUG
          printf("*** DEBUG INFO: %i\n", ++debug);
#endif
          *nreturn_vals = 12;

          values[0].data.d_status = STATUS_SUCCESS;
          values[1].type = PARAM_STRING;
          values[1].data.d_string = fullname;
          values[2].type = PARAM_STRING;
          values[2].data.d_string = filepath;
          values[3].type = PARAM_STRING;
          values[3].data.d_string = filename;
          values[4].type = PARAM_STRING;
          values[4].data.d_string = fileext;
          values[5].type = PARAM_STRING;
          values[5].data.d_string = size;
          values[6].type = PARAM_STRING;
          values[6].data.d_string = mode;
          values[7].type = PARAM_STRING;
          values[7].data.d_string = user;
          values[8].type = PARAM_STRING;
          values[8].data.d_string = group;
          values[9].type = PARAM_STRING;
          values[9].data.d_string = atime;
          values[10].type = PARAM_STRING;
          values[10].data.d_string = mtime;
          values[11].type = PARAM_STRING;
          values[11].data.d_string = ctime;

#ifdef DEBUG
          printf("*** DEBUG INFO: %i\n", ++debug);
#endif
          if(run_mode == RUN_INTERACTIVE) {     // interactive
              g_message("Filename: %s\nPath: %s\nName: %s\nExtension: %s\nSize: %s byte(s)\nOwner: %s\nGroup: %s\nAccess Mode: %s\nChange time: %s\nModif time: %s\nAccess time: %s",
                        fullname, filepath, filename, fileext, size, user, group, mode, ctime, mtime, atime);
          }
      }
      else {
#ifdef DEBUG
          printf("*** DEBUG INFO: %i\n", ++debug);
#endif
          g_message("Image has not filename yet\nSAVE IT FIRST!\n");
          values[0].data.d_status = STATUS_EXECUTION_ERROR;
      }
  }
  else
      g_assert (FALSE);
}
